use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
37050201
Alders
Alders'
Anwar
Atria
Boris
Choroba
Cyrill
Däppen
Heumann
Joaquín
José
Manni
Mohammad
Novgorodcev
Olaf
Open
This
bdaeppen
choroba
cynovg
github
jjatria
lib
libs
mcneb10
mohammad
olaf
ot
script
